package com.sun.org.apache.bcel.internal.classfile;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;

public final class ExceptionTable extends Attribute
{
    private int[] exception_index_table;
    
    public ExceptionTable(ExceptionTable c) {
        this(c.getNameIndex(), c.getLength(), c.getExceptionIndexTable(), c.getConstantPool());
    }
    
    public ExceptionTable(int name_index, int length, int[] exception_index_table, ConstantPool constant_pool) {
        super((byte)3, name_index, length, constant_pool);
        this.exception_index_table = ((exception_index_table != null) ? exception_index_table : new int[0]);
    }
    
    ExceptionTable(int name_index, int length, DataInput input, ConstantPool constant_pool) throws IOException {
        this(name_index, length, (int[])null, constant_pool);
        int number_of_exceptions = input.readUnsignedShort();
        this.exception_index_table = new int[number_of_exceptions];
        for (int i = 0; i < number_of_exceptions; ++i) {
            this.exception_index_table[i] = input.readUnsignedShort();
        }
    }
    
    @Override
    public void accept(Visitor v) {
        v.visitExceptionTable(this);
    }
    
    @Override
    public final void dump(DataOutputStream file) throws IOException {
        super.dump(file);
        file.writeShort(this.exception_index_table.length);
        for (int index : this.exception_index_table) {
            file.writeShort(index);
        }
    }
    
    public final int[] getExceptionIndexTable() {
        return this.exception_index_table;
    }
    
    public final int getNumberOfExceptions() {
        return (this.exception_index_table == null) ? 0 : this.exception_index_table.length;
    }
    
    public final String[] getExceptionNames() {
        String[] names = new String[this.exception_index_table.length];
        for (int i = 0; i < this.exception_index_table.length; ++i) {
            names[i] = super.getConstantPool().getConstantString(this.exception_index_table[i], (byte)7).replace('/', '.');
        }
        return names;
    }
    
    public final void setExceptionIndexTable(int[] exception_index_table) {
        this.exception_index_table = ((exception_index_table != null) ? exception_index_table : new int[0]);
    }
    
    @Override
    public final String toString() {
        StringBuilder buf = new StringBuilder();
        buf.append("Exceptions: ");
        for (int i = 0; i < this.exception_index_table.length; ++i) {
            String str = super.getConstantPool().getConstantString(this.exception_index_table[i], (byte)7);
            buf.append(Utility.compactClassName(str, false));
            if (i < this.exception_index_table.length - 1) {
                buf.append(", ");
            }
        }
        return buf.toString();
    }
    
    @Override
    public Attribute copy(ConstantPool _constant_pool) {
        ExceptionTable c = (ExceptionTable)this.clone();
        if (this.exception_index_table != null) {
            c.exception_index_table = new int[this.exception_index_table.length];
            System.arraycopy(this.exception_index_table, 0, c.exception_index_table, 0, this.exception_index_table.length);
        }
        c.setConstantPool(_constant_pool);
        return c;
    }
}
